import os

from cpk.constants import CPK_CONFIG_DIR
from cpk.types import CPKConfiguration
from cpk.utils.machine import load_machines


cpkconfig: CPKConfiguration = CPKConfiguration(
    path=CPK_CONFIG_DIR,
    machines=load_machines(os.path.join(CPK_CONFIG_DIR, "machines"))
)

__all__ = [
    "cpkconfig"
]



import argparse
from typing import Optional

from cpk import cpkconfig
from cpk.cli import AbstractCLICommand, cpklogger
from cpk.types import Machine, Arguments


class CLIMachineListCommand(AbstractCLICommand):

    KEY = 'machine list'

    @staticmethod
    def parser(parent: Optional[argparse.ArgumentParser] = None,
               args: Optional[Arguments] = None) -> argparse.ArgumentParser:
        parser = argparse.ArgumentParser(parents=[parent], add_help=False)
        # ---
        return parser

    @staticmethod
    def execute(_: Machine, parsed: argparse.Namespace) -> bool:
        machines = cpkconfig.machines
        if len(machines) == 0:
            cpklogger.info("No machines found")
            return True
        # print table
        cpklogger.info(f"Number of machines found: {len(machines)}")
        table_fmt = "   |    {:<8} {:<15} {:<10} {:<1}"
        print("-" * (8 + 15 + 10 + 40))
        print(table_fmt.format('ID', 'NAME', 'TYPE', 'ENDPOINT'))
        for i, (name, machine) in enumerate(machines.items()):
            print(table_fmt.format(i+1, name, machine.type, machine.base_url))
            print()
        # ---
        return True



import argparse
from typing import Optional

from cpk.cli.commands.info import CLIInfoCommand

from cpk import CPKProject

from .. import AbstractCLICommand, cpklogger
from ...types import Machine, Arguments


class CLICleanCommand(AbstractCLICommand):

    KEY = 'clean'

    @staticmethod
    def parser(parent: Optional[argparse.ArgumentParser] = None,
               args: Optional[Arguments] = None) -> argparse.ArgumentParser:
        parser = argparse.ArgumentParser(parents=[parent])
        return parser

    @staticmethod
    def execute(machine: Machine, parsed: argparse.Namespace) -> bool:
        # get project
        project = CPKProject(parsed.workdir, parsed=parsed)

        # show info about project
        CLIInfoCommand.execute(machine, parsed)

        # pick right value of `arch` given endpoint
        if parsed.arch is None:
            cpklogger.info("Parameter `arch` not given, will resolve it from the endpoint.")
            parsed.arch = machine.get_architecture()
            cpklogger.info(f"Parameter `arch` automatically set to `{parsed.arch}`.")

        # create docker client
        docker = machine.get_client()

        # TODO: perform clean

        return True



import os

from .null import NullAdapter


class FileSystemAdapter(NullAdapter):

    def __init__(self, path: str):
        super(FileSystemAdapter, self).__init__('fs')
        self._path = path
        self.enabled = True

    @property
    def name(self) -> str:
        return os.path.basename(self._path).lower()



import argparse
import copy
import dataclasses
import os
import re
import json
from pathlib import Path
from typing import Any, Dict, Union, List, Optional

import jsonschema
import requests

from docker.errors import APIError, ImageNotFound

from .adapters import FileSystemAdapter, ProjectFileAdapter, GitRepositoryAdapter, CLIAdapter
from .adapters.generic import ProjectVersion, GenericAdapter
from .constants import DOCKERHUB_API_URL
from .exceptions import NotACPKProjectException, InvalidCPKProjectFile, \
    CPKProjectSchemaNotSupported, CPKMissingResourceException
from .schemas import get_project_schema
from .types import CPKProjectInfo, GitRepository, CPKTemplateInfo, CPKFileMapping, Machine
from .utils.docker import DEFAULT_REGISTRY
from .utils.git import get_repo_info
from .utils.misc import assert_canonical_arch, parse_configurations, cpk_label


class CPKProject:
    must_have_files = {
        "Dockerfile"
    }
    must_have_directories = set()

    def __init__(self, path: str, parsed: argparse.Namespace = None):
        # make sure the directory exists
        self._path = os.path.abspath(path)
        if not os.path.isdir(self._path):
            raise NotACPKProjectException(self._path)
        # try to read the project info from the project file
        self._info = self._get_info(self._path)
        # validate structure
        self._validate_project_structure()
        # define adapters
        self._adapters = {
            0: GenericAdapter("generic"),
            10: FileSystemAdapter(self._path),
            20: ProjectFileAdapter(self._info),
            30: GitRepositoryAdapter(self._path),
            40: CLIAdapter(parsed)
        }
        # look for a git repo
        self._repo = get_repo_info(self._path)
        # find features
        self._features = CPKProjectFeatures.from_project(self)

    @property
    def path(self) -> str:
        return self._path

    @property
    def name(self) -> str:
        return self._from_adapters("name")

    @property
    def registry(self) -> str:
        return self._from_adapters("registry")

    @property
    def organization(self) -> str:
        return self._from_adapters("organization")

    @property
    def description(self) -> Optional[str]:
        return self._from_adapters("description")

    @property
    def maintainer(self) -> Optional[str]:
        return self._from_adapters("maintainer")

    @property
    def version(self) -> ProjectVersion:
        return self._from_adapters("version")

    @property
    def template(self) -> CPKTemplateInfo:
        return self._info.template

    @property
    def repository(self) -> GitRepository:
        return self._repo

    @property
    def url(self) -> Optional[str]:
        return self._from_adapters("url")

    @property
    def adapters(self) -> Dict[str, GenericAdapter]:
        return {a.id: a for a in self._adapters.values() if a.enabled}

    @property
    def features(self) -> 'CPKProjectFeatures':
        return self._features

    @property
    def mappings(self) -> List[CPKFileMapping]:
        project_maps = self._info.mappings
        template_maps = []
        # project mappings have the priority over template mappings,
        # keep only the template mappings whose destination does not collide with any
        # of the project mappings
        for tmapping in self._info.template.mappings:
            collision = False
            for pmapping in project_maps:
                if pmapping.destination == tmapping.destination:
                    collision = True
                    break
            if not collision:
                template_maps.append(tmapping)
        # replace placeholders in mappings
        placeholders = {
            "project_name": self.name
        }
        mappings = []
        for mapping in template_maps + project_maps:
            cmapping = copy.deepcopy(mapping)
            cmapping.source = mapping.source.format(**placeholders)
            cmapping.destination = mapping.destination.format(**placeholders)
            mappings.append(cmapping)
        # ---
        return mappings

    def resource(self, resource: str) -> str:
        return os.path.join(self.path, resource.lstrip('/'))

    def is_release(self) -> bool:
        if not self.is_clean():
            return False
        if self.version.head is None:
            return False
        return True

    def is_clean(self) -> bool:
        return self._repo.index.clean

    def is_dirty(self) -> bool:
        return not self.is_clean()

    def is_detached(self) -> bool:
        return self._repo.detached

    def image(self, arch: str, docs: bool = False) -> str:
        assert_canonical_arch(arch)
        docs = "-docs" if docs else ""
        version = re.sub(r"[^\w\-.]", "-", self.version.tag)
        image = f"{self.registry}/{self.organization}/{self.name}:{version}{docs}-{arch}"
        return image.lower()

    def image_release(self, arch: str, docs: bool = False, avoid_defaults: bool = False) -> str:
        if not self.is_release():
            raise ValueError("The project repository is not in a release state")
        assert_canonical_arch(arch)
        docs = "-docs" if docs else ""
        version = re.sub(r"[^\w\-.]", "-", self.version.head)
        image = f"{self.organization}/{self.name}:{version}{docs}-{arch}"
        registry = self.registry
        if not avoid_defaults or registry != DEFAULT_REGISTRY:
            image = f"{self.registry}/{image}"
        return image.lower()

    def configurations(self) -> dict:
        configurations = {}
        configurations_file = os.path.join(self._path, "configurations.yaml")
        if os.path.isfile(configurations_file):
            configurations = parse_configurations(configurations_file)
        # ---
        return configurations

    def configuration(self, name: str) -> dict:
        configurations = self.configurations()
        if name not in configurations:
            raise KeyError(f"Configuration with name '{name}' not found.")
        return configurations[name]

    def label(self, key: Union[List[str], str]) -> str:
        if isinstance(key, (list, tuple)):
            key = ".".join(key)
        return cpk_label(f"project.{self.organization}.{self.name}.{key}")

    def build_labels(self) -> Dict[str, str]:
        return {
            self.label("code.vcs"): "git" if self.repository.present else "ND",
            self.label("code.version.tag"): self.version.tag or "ND",
            self.label("code.version.head"): self.version.head or "ND",
            self.label("code.version.closest"): self.version.closest or "ND",
            self.label("code.version.sha"): self.version.sha if (self.version.sha and
                                                                 self.is_clean()) else "ND",
            self.label("code.vcs.repository"): self.repository.name or "ND",
            self.label("code.vcs.branch"): self.repository.branch or "ND",
            self.label("code.vcs.url"): self.repository.origin.url_https or "ND",
            self.label("template.name"): self.template.name,
            self.label("template.version"): self.template.version,
            self.label("template.url"): self.template.url or "ND",
            **self._launchers_labels(),
            **self._configurations_labels()
        }

    def launchers(self) -> List[str]:
        launchers_dir = os.path.join(self.path, "launchers")
        files = (
            [
                os.path.join(launchers_dir, f)
                for f in os.listdir(launchers_dir)
                if os.path.isfile(os.path.join(launchers_dir, f))
            ]
            if os.path.isdir(launchers_dir)
            else []
        )

        def _has_shebang(f):
            with open(f, "rt") as fin:
                return fin.readline().startswith("#!")

        launchers = [Path(f).stem for f in files if os.access(f, os.X_OK) or _has_shebang(f)]
        return launchers

    def image_metadata(self, machine: Machine, arch: str):
        image_name = self.image(arch)
        try:
            image = machine.get_client().images.get(image_name)
            return image.attrs
        except (APIError, ImageNotFound):
            return None

    def image_labels(self, machine: Machine, arch: str):
        image_name = self.image(arch)
        try:
            image = machine.get_client().images.get(image_name)
            return image.labels
        except (APIError, ImageNotFound):
            return None

    def remote_image_metadata(self, arch: str):
        assert_canonical_arch(arch)
        image = f"{self.organization}/{self.name}"
        tag = f"{self.version.tag}-{arch}"
        return self.inspect_remote_image(image, tag)

    def _from_adapters(self, key: str, default: Any = None) -> Any:
        for priority in sorted(self._adapters.keys(), reverse=True):
            adapter = self._adapters[priority]
            if not adapter.enabled:
                continue
            value = getattr(adapter, key)
            if value is not None:
                return value
        return default

    def _validate_project_structure(self):
        must_have_files = {
            "project": self.must_have_files,
            "template": self.template.must_have['files']
        }
        must_have_directories = {
            "project": self.must_have_directories,
            "template": self.template.must_have['directories']
        }
        # check "must_have" files
        for owner, files in must_have_files.items():
            for file in files:
                if not os.path.isfile(self.resource(file)):
                    msg = f"CPK {owner} requires the following file, but this was not found."
                    raise CPKMissingResourceException(file, explanation=msg)
        # check "must_have" directories
        for owner, directories in must_have_directories.items():
            for directory in directories:
                if not os.path.isdir(self.resource(directory)):
                    msg = f"CPK {owner} requires the following directory, but this was not found."
                    raise CPKMissingResourceException(directory, explanation=msg)

    def _launchers_labels(self) -> Dict[str, str]:
        return {self.label("code.launchers"): ",".join(self.launchers())}

    def _configurations_labels(self) -> Dict[str, str]:
        labels = {}
        # add configuration labels
        for cfg_name, cfg_data in self.configurations().items():
            labels[self.label(f"configuration.{cfg_name}")] = json.dumps(cfg_data)
        # ---
        return labels

    @staticmethod
    def _get_info(path: str) -> CPKProjectInfo:
        metafile = os.path.join(path, "project.cpk")
        # check if the file 'project.cpk' is missing
        if not os.path.exists(metafile):
            raise NotACPKProjectException(path)
        # load 'project.cpk'
        with open(metafile, "rt") as fin:
            try:
                data = json.load(fin)
            except json.JSONDecodeError as e:
                raise InvalidCPKProjectFile(path, f"File `{metafile}` must contain a valid JSON. "
                                                  f"DecoderError: {str(e)}")
        # make sure the `schema` field is there
        if 'schema' not in data:
            raise InvalidCPKProjectFile(path, "Missing field: `schema`")
        # make sure we support that schema
        try:
            schema = get_project_schema(data["schema"])
        except FileNotFoundError:
            raise CPKProjectSchemaNotSupported(data["schema"])
        # validate metadata against its declared schema
        try:
            jsonschema.validate(data, schema=schema)
        except jsonschema.exceptions.ValidationError as e:
            raise InvalidCPKProjectFile(path, str(e))
        # parse template
        template = CPKTemplateInfo.from_template_dict(data['template']) \
            if data.get("template", None) else CPKTemplateInfo.from_project_path(path)
        # metadata is valid
        return CPKProjectInfo(
            name=data["name"],
            organization=data.get("organization", None),
            description=data.get("description", None),
            maintainer=data.get("maintainer", None),
            template=template,
            version=data.get("version", None),
            registry=data.get("registry", None),
            tag=data.get("tag", None),
            mappings=list(map(lambda m: CPKFileMapping.from_dict(m), data.get('mappings', [])))
        )

    @staticmethod
    def inspect_remote_image(image, tag):
        res = requests.get(DOCKERHUB_API_URL["token"].format(image=image)).json()
        token = res["token"]
        # ---
        res = requests.get(
            DOCKERHUB_API_URL["digest"].format(image=image, tag=tag),
            headers={
                "Accept": "application/vnd.docker.distribution.manifest.v2+json",
                "Authorization": "Bearer {0}".format(token),
            },
        ).text
        digest = json.loads(res)["config"]["digest"]
        # ---
        res = requests.get(
            DOCKERHUB_API_URL["inspect"].format(image=image, tag=tag, digest=digest),
            headers={"Authorization": "Bearer {0}".format(token)},
        ).json()
        return res


@dataclasses.dataclass
class CPKProjectFeatures:
    launchers: bool
    setup: bool
    configurations: bool
    assets: bool

    @staticmethod
    def from_project(project: CPKProject) -> 'CPKProjectFeatures':
        return CPKProjectFeatures(
            launchers=os.path.isdir(os.path.join(project.path, 'launchers')),
            setup=os.path.isfile(os.path.join(project.path, 'setup.sh')),
            configurations=os.path.isfile(os.path.join(project.path, 'configurations.yaml')),
            assets=os.path.isdir(os.path.join(project.path, 'assets'))
        )



import logging
import os
import shutil
import subprocess
import time
from shutil import which
from typing import Optional, Union

import docker
from cpk.exceptions import CPKException
from docker import DockerClient

from cpk.types import Machine

from cryptography.hazmat.primitives import serialization as crypto_serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.backends import default_backend as crypto_default_backend
from sshconf import empty_ssh_config_file

from cpk.utils.misc import configure_ssh_for_cpk


class FromEnvMachine(Machine):

    def __init__(self):
        super(FromEnvMachine, self).__init__("from-environment")

    @property
    def is_local(self) -> bool:
        return True

    def get_client(self) -> DockerClient:
        return docker.from_env()


class TCPMachine(Machine):
    type: str = "tcp"

    def __init__(self, name: str, host: str, port: Optional[Union[int, str]] = None):
        if port is not None:
            host = f"{host}:{port}"
        super(TCPMachine, self).__init__(name, host)

    @property
    def is_local(self) -> bool:
        hostname, *_ = self._base_url.split(":")
        return hostname in ["tcp://localhost", "tcp://127.0.0.1", "tcp://127.0.1.1"]

    def get_client(self) -> DockerClient:
        try:
            return docker.DockerClient(base_url=self.base_url)
        except docker.errors.DockerException as e:
            raise CPKException(str(e))


class UnixSocketMachine(TCPMachine):
    type: str = "socket"

    def __init__(self, name: str, host: str = "unix:///var/run/docker.sock"):
        super(UnixSocketMachine, self).__init__(name, host)

    @property
    def is_local(self) -> bool:
        return True


class SSHMachine(Machine):
    type: str = "ssh"

    def __init__(self, name: str, user: str, host: str, port: Optional[Union[str, int]] = 22):
        config = {
            "user": user,
            "host": host,
            "port": int(port) if port is not None else 22,
        }
        super(SSHMachine, self).__init__(name, configuration=config)

    @property
    def is_local(self) -> bool:
        return False

    @property
    def user(self) -> str:
        return self._configuration["user"]

    @property
    def host(self) -> str:
        return self._configuration["host"]

    @property
    def port(self) -> Optional[int]:
        return self._configuration["port"]

    @property
    def uri(self) -> Optional[str]:
        cfg = self._configuration
        return f"{cfg['user']}@{cfg['host']}"

    @property
    def base_url(self) -> Optional[str]:
        return f"ssh://{self.uri}:{self.port}"

    def get_client(self) -> DockerClient:
        return docker.DockerClient(base_url=self.base_url, use_ssh_client=True)

    def save(self, logger: Optional[logging.Logger] = None):
        from cpk import cpkconfig
        path = os.path.join(cpkconfig.path, "machines", self.name)
        # make sure the tool `ssh-copy-id` is present
        ssh_copy_id = which("ssh-copy-id")
        if ssh_copy_id is None:
            raise ValueError("The SSH command 'ssh-copy-id' is required.")
        if logger:
            logger.debug(f"Tool 'ssh-copy-id' found at '{ssh_copy_id}'")
        # store machine descriptor
        super(SSHMachine, self).save()
        # create keys
        key = rsa.generate_private_key(
            backend=crypto_default_backend(),
            public_exponent=65537,
            key_size=2048
        )
        # noinspection PyTypeChecker
        private_key = key.private_bytes(
            crypto_serialization.Encoding.PEM,
            crypto_serialization.PrivateFormat.TraditionalOpenSSL,
            crypto_serialization.NoEncryption()
        )
        # noinspection PyTypeChecker
        public_key = key.public_key().public_bytes(
            crypto_serialization.Encoding.OpenSSH,
            crypto_serialization.PublicFormat.OpenSSH
        )
        # save keys
        ssh_keys_dir = os.path.join(path, "keys")
        if logger:
            logger.debug(f"Creating keys directory '{ssh_keys_dir}'")
        os.makedirs(ssh_keys_dir, mode=0o700, exist_ok=True)
        # - private key
        private_key_fpath = os.path.join(ssh_keys_dir, "id_rsa")
        with open(os.open(private_key_fpath, os.O_CREAT | os.O_WRONLY, 0o600), "wb") as fout:
            if logger:
                logger.debug(f"Writing private key to '{private_key_fpath}'")
            fout.write(private_key)
        # - public key
        public_key_fpath = os.path.join(ssh_keys_dir, "id_rsa.pub")
        with open(os.open(public_key_fpath, os.O_CREAT | os.O_WRONLY, 0o644), "wb") as fout:
            if logger:
                logger.debug(f"Writing public key to '{private_key_fpath}'")
            fout.write(public_key)
        # create SSH configuration file
        ssh_conf_fpath = os.path.join(path, "host.conf")
        config = empty_ssh_config_file()
        config.add(
            self.host,
            Hostname=self.host,
            User=self.user,
            **({"Port": self.port} if self.port else {}),
            IdentityFile=private_key_fpath
        )
        if logger:
            logger.debug(f"Writing SSH host configuration to '{ssh_conf_fpath}'")
        config.write(ssh_conf_fpath)
        # configure user's environment for cpk
        configure_ssh_for_cpk(logger)
        # try to transfer the keys to the machine
        cmd = ["ssh-copy-id", "-i", private_key_fpath, self.uri]
        if logger:
            logger.debug(f"Running $> '{str(cmd)}'")
            logger.info("The newly created RSA key will be copied to the destination machine.")
            logger.info("Enter the SSH password when prompted:")
            time.sleep(0.5)
        retcode = subprocess.call(cmd, stderr=subprocess.DEVNULL)
        if retcode == 0:
            return
        self.remove(logger=logger)
        raise RuntimeError("RSA keys could not be copied to the destination machine.")

    def __enter__(self):
        from cpk import cpkconfig
        # create SSH host configuration file in the SSH pool directory .cpk/ssh
        cpk_ssh_pool_dir = os.path.join(cpkconfig.path, "ssh")
        host_cfg_src_fpath = os.path.join(self.config_path, "host.conf")
        host_cfg_dest_fpath = os.path.join(cpk_ssh_pool_dir, f"{self.name}.conf")
        shutil.copy(host_cfg_src_fpath, host_cfg_dest_fpath)

    def __exit__(self, exc_type, exc_val, exc_tb):
        from cpk import cpkconfig
        # remove SSH host configuration file from the SSH pool directory .cpk/ssh
        cpk_ssh_pool_dir = os.path.join(cpkconfig.path, "ssh")
        host_cfg_fpath = os.path.join(cpk_ssh_pool_dir, f"{self.name}.conf")
        if os.path.exists(host_cfg_fpath) and os.path.isfile(host_cfg_fpath):
            os.remove(host_cfg_fpath)



import argparse
from typing import Optional

from docker.errors import APIError

from .endpoint import CLIEndpointInfoCommand
from .info import CLIInfoCommand
from .. import AbstractCLICommand, cpklogger
from ... import CPKProject
from ...exceptions import CPKProjectPushException
from ...types import Machine, Arguments
from ...utils.cli import check_git_status
from ...utils.docker import push_image


class CLIPushCommand(AbstractCLICommand):

    KEY = 'push'

    @staticmethod
    def parser(parent: Optional[argparse.ArgumentParser] = None,
               args: Optional[Arguments] = None) -> argparse.ArgumentParser:
        parser = argparse.ArgumentParser(parents=[parent])
        parser.add_argument(
            "--rm",
            default=False,
            action="store_true",
            help="Remove the images once the build is finished",
        )
        parser.add_argument(
            "--tag",
            default=None,
            type=str,
            help="Custom tag"
        )
        return parser

    @staticmethod
    def execute(machine: Machine, parsed: argparse.Namespace) -> bool:
        # get project
        project = CPKProject(parsed.workdir, parsed=parsed)

        # show info about project
        CLIInfoCommand.execute(machine, parsed)

        # check git workspace status
        proceed = check_git_status(project, parsed)
        if not proceed:
            return False

        # get info about docker endpoint
        CLIEndpointInfoCommand.execute(machine, parsed)

        # pick right value of `arch` given endpoint
        if parsed.arch is None:
            cpklogger.info("Parameter `arch` not given, will resolve it from the endpoint.")
            parsed.arch = machine.get_architecture()
            cpklogger.info(f"Parameter `arch` automatically set to `{parsed.arch}`.")

        # create defaults
        image = project.image(parsed.arch)

        # print info about multiarch
        msg = "Pushing image {} to {}.".format(image, project.registry)
        cpklogger.info(msg)

        # push image
        try:
            push_image(machine, image, progress=True)
        except APIError as e:
            cpklogger.error(f"An error occurred while pushing the project image:\n{str(e)}")
            return False
        except CPKProjectPushException:
            cpklogger.error(f"An error occurred while building the project image.")
            return False

        cpklogger.info("Image pushed successfully!")



import glob
import os
import sys
from distutils.core import setup
from typing import List


def get_version(filename):
    import ast
    version = None
    with open(filename) as f:
        for line in f:
            if line.startswith('__version__'):
                version = ast.parse(line).body[0].value.s
                break
        else:
            raise ValueError('No version found in %r.' % filename)
    if version is None:
        raise ValueError(filename)
    return version


def _get_all_files(parent: str, child: str) -> List[str]:
    path = os.path.abspath(os.path.join(parent, child))
    hidden = glob.glob(os.path.join(path, ".**"), recursive=True)
    nonhidden = glob.glob(os.path.join(path, "**"), recursive=True)
    items = hidden + nonhidden
    files = filter(os.path.isfile, items)
    files = list(map(lambda p: os.path.relpath(p, parent), files))
    return files


def get_decorator_files() -> List[str]:
    this_file = os.path.abspath(__file__)
    cpk_dir = os.path.join(os.path.dirname(this_file), "include", "cpk")
    return _get_all_files(cpk_dir, "decorator")


def get_skeleton_files() -> List[str]:
    this_file = os.path.abspath(__file__)
    cpk_dir = os.path.join(os.path.dirname(this_file), "include", "cpk")
    return _get_all_files(cpk_dir, "skeleton")


if sys.version_info < (3, 6):
    msg = 'cpk works with Python 3.6 and later.\nDetected %s.' % str(sys.version)
    sys.exit(msg)

lib_version = get_version(filename='include/cpk/__init__.py')

setup(
    name='cpk',
    packages=[
        'cpk',
        'cpk.adapters',
        'cpk.cli',
        'cpk.cli.commands',
        'cpk.cli.commands.machine',
        'cpk.cli.commands.endpoint',
        'cpk.utils'
    ],
    package_dir={
        'cpk': 'include/cpk'
    },
    package_data={
        "cpk": [
            "schemas/*/*.json",
            *get_decorator_files(),
            *get_skeleton_files()
        ],
    },
    version=lib_version,
    license='MIT',
    description='Toolkit that standardize the way code in a project is structured and packaged '
                'for maximum portability, readability and maintainability.',
    author='Andrea F. Daniele',
    author_email='afdaniele@ttic.edu',
    url='https://github.com/afdaniele/cpk',
    download_url='https://github.com/afdaniele/cpk/tarball/{}'.format(lib_version),
    zip_safe=False,
    include_package_data=True,
    keywords=['code', 'container', 'containerization', 'package', 'toolkit', 'docker'],
    install_requires=[
        'docker>=4.4.0',
        'requests',
        'jsonschema',
        'termcolor',
        'pyyaml',
        'sshconf',
        'cryptography',
        'x-docker>=0.0.2',
        *(['dataclasses'] if sys.version_info < (3, 7) else [])
    ],
    scripts=[
        'include/cpk/bin/cpk'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
